

#include <windows.h>
#include <stdio.h>		// printf() / scanf()
#define _USE_MATH_DEFINES
#include <math.h>
#include <limits.h>

#include <io.h>
#include <string>
#include <vector>
#include <map>

#include "filter.h"
#include "scanline.h"

static inline int modulo(int a, int b) {
	int m = a % b;

	if (m >= 0) {
		return m;
	} else {
		return m + b;
	}
}

static char *g_dllpath = NULL;
static std::map<std::string, int> g_trackMap;
static std::map<std::string, int> g_checkMap;
static std::vector<int> g_bef_check;

static int getTrackVal(FILTER *fp, char *str) {
	char errmsg[8192];

	std::map<std::string, int>::iterator itr;
	itr = g_trackMap.find(str);

	if (itr == g_trackMap.end()) {
		sprintf_s(errmsg, "track %s ݂͑܂", str);
		MessageBox(fp->hwnd, errmsg, "G[", MB_OK);
		return 0;
	}

	int val = fp->track[itr->second];
	val = std::max<int>(val, fp->track_s[itr->second]);
	val = std::min<int>(val, fp->track_e[itr->second]);

	return val;
}

static int getCheckVal(FILTER *fp, char *str) {
	char errmsg[8192];

	std::map<std::string, int>::iterator itr;
	itr = g_checkMap.find(str);

	if (itr == g_checkMap.end()) {
		sprintf_s(errmsg, "check %s ݂͑܂", str);
		MessageBox(fp->hwnd, errmsg, "G[", MB_OK);
		return 0;
	}

	return fp->check[itr->second];
}

void initFilter(char *dllpath, FILTER_DLL *fdp) {
	g_dllpath = dllpath;

	g_trackMap.clear();
	for (int i=0; i<fdp->track_n; i++) {
		g_trackMap[fdp->track_name[i]] = i;
	}

	g_checkMap.clear();
	for (int i=0; i<fdp->check_n; i++) {
		g_checkMap[fdp->check_name[i]] = i;
	}
	g_bef_check.resize(fdp->check_n, 0);
}

void updateFilter(FILTER *fp) {
}


void exitFilter() {
}

BOOL execFilter(FILTER *fp,FILTER_PROC_INFO *fpip) {
	PIXEL_YC *buffer0 = fpip->ycp_edit;
	PIXEL_YC *buffer1 = fpip->ycp_temp;

	int mw = fpip->max_w;
	int mh = fpip->max_h;
	int w = fpip->w;
	int h = fpip->h;

	int sw1 = getTrackVal(fp, "1");
	int sw2 = getTrackVal(fp, "2");
	int sy1 = getTrackVal(fp, "Px1");
	int sy2 = getTrackVal(fp, "Px2");
	int sft = getTrackVal(fp, "ڂ");

	int sw = sw1 + sw2;
	sft = std::min<int>(sft, sw);

	std::vector<int> mytbl_org(sw);
	std::vector<int> mytbl(sw);

	for(int i=0; i<sw; i++){
		mytbl_org[i] = (i < sw1)? sy1 : sy2;
	}

	for (int i=0; i<sw; i++) {
		int sum = 0;
		for (int j = -sft; j <= sft; j++) {
			sum += mytbl_org[modulo(i+j, sw)];
		}
		mytbl[i] = sum / (sft * 2 + 1);
	}


	for (int iy = 0; iy < h; iy++) {
		int my = mytbl[iy % sw];
		int src = iy * mw;

		for (int ix = 0; ix < w; ix++) {
			PIXEL_YC pix = buffer0[src];

			pix.y  = (pix.y * my) >> 8;
			pix.cb = (pix.cb * my) >> 8;
			pix.cr = (pix.cr * my) >> 8;

			buffer0[src] = pix;
			src++;
		}
	}

	return TRUE;
}
